@@ -1,6 +1,40 @@
 use crate::*;
 use std::sync::{Arc, RwLock};
 
+/// Parses and then ignores the result, only advancing `offset`.
+pub struct IgnoreParser<A> {
+  pub parser: Arc<RwLock<dyn Parser<Output = A>>>,
+}
+impl<A> IgnoreParser<A> {
+  pub fn new(parser: &Arc<RwLock<dyn Parser<Output = A>>>) -> Self {
+    IgnoreParser {
+      parser: Arc::clone(parser),
+    }
+  }
+}
+impl<A> Clone for IgnoreParser<A> {
+  fn clone(&self) -> Self {
+    IgnoreParser {
+      parser: Arc::clone(&self.parser),
+    }
+  }
+}
+impl<A> Parser for IgnoreParser<A> {
+  type Output = ();
+
+  fn parse_unprotected(self: &mut Self, data: &[u8], offset: &mut usize) -> ParseResult<Self::Output> {
+    let parse_result = match self.parser.try_write() {
+      Ok(p) => p,
+      Err(_) => return Err(ParseError::LockedSubparser),
+    }.parse(data, offset);
+    match parse_result {
+      Ok(_result) => Ok(()),
+      Err(err) => Err(err),
+    }
+  }
+}
+
+/// Parses the left parser and then the right parser, and then joins the result in a tuple.
 pub struct AndParser<A, B> {
   pub left_parser: Arc<RwLock<dyn Parser<Output = A>>>,
   pub right_parser: Arc<RwLock<dyn Parser<Output = B>>>,
@@ -39,10 +73,12 @@ impl<A, B> Parser for AndParser<A, B> {
   }
 }
 
+/// A container for either the left result or the right result.
 pub enum OrParserSuccess<A, B> {
   Left(A),
   Right(B),
 }
+/// Parses the left parser and then the right parser, returning the first successful result.
 pub struct OrParser<A, B> {
   pub left_parser: Arc<RwLock<dyn Parser<Output = A>>>,
   pub right_parser: Arc<RwLock<dyn Parser<Output = B>>>,
@@ -86,6 +122,7 @@ impl<A, B> Parser for OrParser<A, B> {
   }
 }
 
+/// Repeatedly applies a parser over a given range.
 pub struct RangeParser<A> {
   pub parser: Arc<RwLock<dyn Parser<Output = A>>>,
   pub range: std::ops::Range<usize>,
@@ -143,6 +180,7 @@ impl<A> Parser for RangeParser<A> {
   }
 }
 
+/// Parses without advancing `offset`, returning `None` on error.
 pub struct PeekParser<A> {
   pub parser: Arc<RwLock<dyn Parser<Output = A>>>,
 }
